@@ -6,9 +6,12 @@ use crate::Registry;
 use crate::server::{self, Request, Response, HttpError};
 
 pub async fn handle_request(registry: Arc<RwLock<Registry>>, request: Request, api_path: &str) -> Result<Response, HttpError> {
-	if let Some(api_path) = api_path.strip_prefix("crates/") {
+	if api_path == "crates" {
+		search(registry, request.uri().query())
+	} else if let Some(api_path) = api_path.strip_prefix("crates/") {
 		handle_crate_request(registry, request, api_path).await
 	} else {
+		log::warn!("Got request for unknown or unimplemented API V1 endpoint: {}", api_path);
 		server::not_found()
 	}
 }
@@ -19,16 +22,25 @@ async fn handle_crate_request(registry: Arc<RwLock<Registry>>, request: Request,
 	} else {
 		let (name, rest) = match api_path.split_once('/') {
 			Some(x) => x,
-			None => return server::not_found(),
+			None => {
+				log::warn!("Failed to determine crate name from API url: {}", api_path);
+				return server::not_found();
+			},
 		};
 		let (version, action) = match rest.split_once('/') {
 			Some(x) => x,
-			None => return server::not_found(),
+			None => {
+				log::warn!("Failed to determine crate action from API url: {}", api_path);
+				return server::not_found();
+			},
 		};
 		match action {
 			"yank" => yank_crate(registry, name, version, request.method()),
 			"unyank" => unyank_crate(registry, name, version, request.method()),
-			_ => server::not_found()
+			_ => {
+				log::warn!("Got request for unknown or unimplemented crate action: {}", action);
+				server::not_found()
+			},
 		}
 	}
 }
@@ -37,6 +49,7 @@ async fn publish_crate(registry: Arc<RwLock<Registry>>, request: Request) -> Res
 	use sha2::Digest;
 
 	if let Some(response) = server::check_supported_method(request.method(), &[Method::PUT]) {
+		log::warn!("Unsupported request method for v1/crates/new: {}", request.method());
 		return response;
 	}
 
@@ -224,6 +237,88 @@ fn unyank_crate(registry: Arc<RwLock<Registry>>, name: &str, version: &str, meth
 	}
 }
 
+fn search(registry: Arc<RwLock<Registry>>, url_query: Option<&str>) -> Result<Response, HttpError> {
+	#[derive(serde::Deserialize)]
+	struct Params<'a> {
+		q: Option<&'a str>,
+		per_page: Option<usize>,
+	}
+
+	#[derive(serde::Serialize)]
+	struct FoundCrate {
+		name: String,
+		max_version: String,
+		description: String,
+	}
+
+	#[derive(serde::Serialize)]
+	struct SearchResultsMeta {
+		total: usize,
+	}
+
+	#[derive(serde::Serialize)]
+	struct SearchResults {
+		crates: Vec<FoundCrate>,
+		meta: SearchResultsMeta,
+	}
+
+	let params: Params = match serde_urlencoded::from_str(url_query.unwrap_or("")) {
+		Err(e) => return error_response(e),
+		Ok(params) => params,
+	};
+
+	let query = params.q.unwrap_or("");
+	let max_results = params.per_page.unwrap_or(10);
+
+	let registry = registry.read().unwrap();
+
+	let mut crates: Vec<_> = registry.iter_crate_names()
+		.filter_map(|name| {
+			let name = match name {
+				Ok(x) => x,
+				Err(e) => {
+					log::warn!("{}", e);
+					return None;
+				},
+			};
+			if !name.contains(&query) {
+				return None;
+			}
+			let entries = match registry.read_index(&name) {
+				Ok(x) => x,
+				Err(e) => {
+					log::warn!("{}", e);
+					return None;
+				}
+			};
+
+			entries
+				.iter()
+				.filter_map(|entry| semver::Version::parse(&entry.version).ok())
+				.max_by_key(|version| version.clone())
+				.map(|version| {
+					FoundCrate {
+						name,
+						max_version: version.to_string(),
+						description: "".into(), // TODO: omfg, got to read the compressed crate file to extract the manifest
+					}
+				})
+		})
+		.collect();
+
+	let total = crates.len();
+	crates.truncate(max_results);
+
+	let json = serde_json::to_string(&SearchResults {
+		crates,
+		meta: SearchResultsMeta {
+			total,
+		}
+	}).unwrap();
+
+	json_response(json)
+}
+
 fn error_response(message: impl std::fmt::Display) -> Result<Response, HttpError> {
 	#[derive(serde::Serialize)]
 	struct ErrorResponse {
